#import "NodeMediaClientPlugin.h"
#import "NodeMediaPlatformViewFactory.h"
#import "NodeMediaClient.h"
#import "NodeMediaMethodHandler.h"

@interface NodeMediaClientPlugin() <FlutterStreamHandler>
 @property (nonatomic, strong) id<FlutterPluginRegistrar> registrar;
 @property (nonatomic, strong) FlutterEventChannel *eventChannel;
 @property (nonatomic, strong) FlutterEventSink eventSink;
@end


@implementation NodeMediaClientPlugin
+(void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    
  NodeMediaClientPlugin *instance = [[NodeMediaClientPlugin alloc] init];
  instance.registrar = registrar;
    
  FlutterMethodChannel *methodChannel = [FlutterMethodChannel
      methodChannelWithName:@"node_media_client_plugin"
            binaryMessenger:[registrar messenger]];
    [registrar addMethodCallDelegate:instance channel:methodChannel];
    
    FlutterEventChannel *eventChannel = [FlutterEventChannel eventChannelWithName:@"node_media_client_plugin_event_handler" binaryMessenger:[registrar messenger]];
    [eventChannel setStreamHandler:instance];
    instance.eventChannel = eventChannel;
    
    [registrar registerViewFactory:[NodeMediaPlatformViewFactory sharedInstance] withId:@"node_media_client_view"];
}

- (FlutterError* _Nullable)onListenWithArguments:(id _Nullable)arguments
                                       eventSink:(FlutterEventSink)events {
    self.eventSink = events;
    NSLog(@"[FlutterEventSink] [onListen] set eventSink: %p", _eventSink);
    return nil;
}

- (FlutterError* _Nullable)onCancelWithArguments:(id _Nullable)arguments {
    NSLog(@"[FlutterEventSink] [onCancel] set eventSink: %p to nil", _eventSink);
    self.eventSink = nil;
    return nil;
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    NSLog(@"[DartCall] [%@]", call.method);
    if ([@"createEngine" isEqualToString:call.method]) {

        NSMutableDictionary *argumentsMap = [NSMutableDictionary dictionaryWithDictionary:call.arguments];

        argumentsMap[@"eventSink"] = _eventSink;
        argumentsMap[@"registrar"] = _registrar;

        call = [FlutterMethodCall methodCallWithMethodName:@"createEngine" arguments:argumentsMap];
    }

    SEL selector = NSSelectorFromString([NSString stringWithFormat:@"%@:result:", call.method]);

    // Handle unrecognized method
    if (![[NodeMediaMethodHandler sharedInstance] respondsToSelector:selector]) {
        NSLog(@"[handleMethodCall] Unrecognized selector: %@", call.method);
        result(FlutterMethodNotImplemented);
        return;
    }

    NSMethodSignature *signature = [[NodeMediaMethodHandler sharedInstance] methodSignatureForSelector:selector];

    NSInvocation* invocation = [NSInvocation invocationWithMethodSignature:signature];

    invocation.target = [NodeMediaMethodHandler sharedInstance];
    invocation.selector = selector;

    [invocation setArgument:&call atIndex:2];
    [invocation setArgument:&result atIndex:3];

    [invocation invoke];
}

@end
