//
//  NodeMediaMethodHandler.m
//  Pods
//
//  Created by seven on 2021/2/4.
//

#import "NodeMediaMethodHandler.h"
#import "NodeMediaEngine.h"
#import "NodeMediaClient.h"
#import "NodeMediaPlatformView.h"
#import "NodeMediaPlatformViewFactory.h"
#import "NodeMediaPublishEngine.m"

@interface NodeMediaMethodHandler()
    @property (strong, nonatomic) NodePublisher *np;
@end

@implementation NodeMediaMethodHandler

+ (instancetype)sharedInstance{
    static NodeMediaMethodHandler * instance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        instance = [[NodeMediaMethodHandler alloc] init];
    });
    return instance;
}

#pragma mark - Main

- (void) createEngine: (FlutterMethodCall *)call result:(FlutterResult)result{
    // NodeMediaEventHandler *event = [[NodeMediaEventHandler alloc] init];
    FlutterEventSink sink = call.arguments[@"eventSink"];
    if (!sink) {
        NSLog(@"[createEngine] FlutterEventSink is nil");
    }
    // event.eventSink = sink;
    // [NodeMediaPublishEngine createEngine:@"" eventHandler: event];
    [NodeMediaEventHandler sharedInstance].eventSink = sink;
    [NodeMediaPublishEngine createEngine:@"" eventHandler: [NodeMediaEventHandler sharedInstance]];
}

#pragma mark - Playing

- (void) startPlayingStream: (FlutterMethodCall *)call result:(FlutterResult)result{
    NSString *url = call.arguments[@"url"];
    NSDictionary *canvasMap = call.arguments[@"canvas"];
    int64_t viewID = [canvasMap[@"view"] isKindOfClass:[NSNull class]] ? 0 : [canvasMap[@"view"] longLongValue];
    NodeMediaPlatformView *platformView = [[NodeMediaPlatformViewFactory sharedInstance] getPlatformView:@(viewID)];
    if (platformView) {
        [[NodeMediaEngine sharedEngine] startPlayingStream:platformView.getUIView InputUrl:url];
        NSLog(@"[startPlayingStream] %@", platformView.getUIView);
    } else {
        NSString *errorMessage = [NSString stringWithFormat:@"The PlatformView for viewID:%ld cannot be found, developer should call `createPlatformView` first and get the viewID", (long)viewID];
        NSLog(@"[startPreview] %@", errorMessage);
        result([FlutterError errorWithCode:[@"startPlayingStream_No_PlatformView" uppercaseString] message:errorMessage details:nil]);
        return;
    }
}

- (void) stopPlayingStream: (FlutterMethodCall *)call result:(FlutterResult)result{
    NSLog(@"[stopPlayingStream]");
    [[NodeMediaEngine sharedEngine] stopPlayingStream];
}

#pragma mark - Publishing

- (void) startPreview: (FlutterMethodCall *)call result:(FlutterResult)result{
    // int channel = [call.arguments[@"channel"] isKindOfClass:[NSNull class]] ? 0 : [call.arguments[@"channel"] intValue];
    NSDictionary *canvasMap = call.arguments[@"canvas"];
    int64_t viewID = [canvasMap[@"view"] isKindOfClass:[NSNull class]] ? 0 : [canvasMap[@"view"] longLongValue];
    NodeMediaPlatformView *platformView = [[NodeMediaPlatformViewFactory sharedInstance] getPlatformView:@(viewID)];
    if (platformView) {
        [[NodeMediaPublishEngine sharedEngine] startPreview: platformView.getUIView];
        NSLog(@"[startPreview] %@", platformView.getUIView);
    } else {
        NSString *errorMessage = [NSString stringWithFormat:@"The PlatformView for viewID:%ld cannot be found, developer should call `createPlatformView` first and get the viewID", (long)viewID];
        NSLog(@"[startPreview] %@", errorMessage);
        result([FlutterError errorWithCode:[@"startPreview_No_PlatformView" uppercaseString] message:errorMessage details:nil]);
        return;
    }
}

- (void) startPublishingStream: (FlutterMethodCall *)call result:(FlutterResult)result{
    NSString *url = call.arguments[@"url"];
    NSLog(@"[startPublishingStream] url %@====================================",url);
    [[NodeMediaPublishEngine sharedEngine] startPublishingStream:url];
}

- (void) stopPublishingStream: (FlutterMethodCall *)call result:(FlutterResult)result{
    NSLog(@"[stopPublishingStream]");
    [[NodeMediaPublishEngine sharedEngine] stopPublishingStream];
}

@end
