//
//  NodeMediaPlatformViewFactory.m
//  Pods
//
//  Created by seven on 2021/2/3.
//

#import "NodeMediaPlatformViewFactory.h"


@interface NodeMediaPlatformViewFactory()

@property (nonatomic, strong) NSMutableDictionary<NSNumber*,NodeMediaPlatformView*> *platformViewMap;

@end

@implementation NodeMediaPlatformViewFactory

+ (instancetype)sharedInstance{
    static NodeMediaPlatformViewFactory *instance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        instance = [[NodeMediaPlatformViewFactory alloc] init];
    });
    return instance;
}

- (instancetype)init{
    self = [super init];
    if(self){
        _platformViewMap = [[NSMutableDictionary alloc] init];
    }
    return self;
}

- (NodeMediaPlatformView *)getPlatformView:(NSNumber *)viewID{
    NSLog(@"[NodeMediaPlatformViewFactory] [getPlatformView] viewID:%d", viewID.intValue);
    
    [self logCurrentPlatformViews];
    
    return [self.platformViewMap objectForKey:viewID];
}

- (void) addPlatformView: (NodeMediaPlatformView *) view viewID:(NSNumber *) viewID{
    [self.platformViewMap setObject:view forKey:viewID];
}

-  (BOOL)destoryPlatformView:(NSNumber *)viewID{
    NodeMediaPlatformView *platformView = self.platformViewMap[viewID];
    if(!platformView){
        NSLog(@"[NodeMediaPlatformViewFactory] [destoryPlatformView] platform view for viewID:%d not exists", viewID.intValue);
        return NO;
    }
    
    NSLog(@"[NodeMediaPlatformViewFactory] [destoryPlatformView] viewID:%d , UIView:%p", viewID.intValue, [platformView getUIView]);
    
    [self.platformViewMap removeObjectForKey:viewID];
    
    return YES;
}

- (void)logCurrentPlatformViews {
    NSMutableString *desc = [[NSMutableString alloc] init];
    for (NSNumber *i in self.platformViewMap) {
        NodeMediaPlatformView *eachPlatformView = self.platformViewMap[i];
        if (eachPlatformView) {
            [desc appendFormat:@"[ID:%d|View:%p] ", i.intValue, eachPlatformView.getUIView];
        }
    }
    NSLog(@"[NodeMediaPlatformViewFactory] currentPlatformViews: %@", desc);
}

#pragma mark FlutterPlatformViewFactory Delegate

- (NSObject<FlutterPlatformView> *)createWithFrame:(CGRect)frame viewIdentifier:(int64_t)viewId arguments:(id)args {
    NodeMediaPlatformView *view = [[NodeMediaPlatformView alloc] initWithRect:frame viewID:viewId];
    [self addPlatformView:view viewID:@(viewId)];
     return view;
}

@end
