//
//  NodeMediaPublishEngine.m
//  node_media_client_plugin
//
//  Created by seven on 2021/2/4.
//

#import "NodeMediaPublishEngine.h"

@interface NodeMediaPublishEngine()
 @property (nonatomic, assign) NSString *appSign;
 @property (nonatomic, strong) NodePublisher *publisher;
@end

@implementation NodeMediaPublishEngine

static NodeMediaPublishEngine *instance = nil;

+ (instancetype)createEngine:(NSString *)appSign eventHandler:(id <NodePublisherDelegate>)eventHandler{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        instance = [[NodeMediaPublishEngine alloc] init];
        instance.appSign = appSign;
        instance.publisher = [[NodePublisher alloc] init];
        [instance.publisher setNodePublisherDelegate:eventHandler];
        [instance.publisher setAudioParamBitrate:32*1000 profile:AUDIO_PROFILE_HEAAC];
        [instance.publisher setVideoParamPreset:VIDEO_PPRESET_16X9_360 fps:15 bitrate:500000 profile:VIDEO_PROFILE_MAIN frontMirror:NO];
        [instance.publisher setDenoiseEnable:YES];
        [instance.publisher setBeautyLevel:3];
        [instance.publisher startPreview];
    });
    return instance;
}

+ (instancetype)sharedEngine{
    return instance;
}

- (void)startPreview:(UIView*)preView{
    [_publisher setCameraPreview:preView cameraId:CAMERA_FRONT frontMirror:YES];
}

- (void)startPublishingStream:(NSString*)outputUrl{
    [_publisher setOutputUrl:outputUrl];
    [_publisher start];
}

- (void)stopPublishingStream{
    [_publisher stopPreview];
    [_publisher stop];
}

@end
