import 'dart:async';
import 'dart:convert';

import 'package:flutter/services.dart';

import 'node_media_client_engine.dart';
import 'node_media_defines.dart';

class NodeMediaClientPlugin {
  static const MethodChannel _channel = const MethodChannel('node_media_client_plugin');
  static const EventChannel _event = const EventChannel('node_media_client_plugin_event_handler');

  /// Used to receive the native event stream
  static StreamSubscription<dynamic> _streamSubscription;

  NodeMediaClientPlugin._internal();

  static final NodeMediaClientPlugin instance =
      NodeMediaClientPlugin._internal();

  static MethodChannel get methodChannel => _channel;

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  static Future<void> createEngine() async {
    _registerEventHandler();
    await _channel.invokeMethod('createEngine');
  }

  static Future<void> destroyEngine() async {
    await _channel.invokeMethod('destroyEngine');
    _unregisterEventHandler();
  }

  Future<void> startPreview({NDCanvas canvas}) async {
    return await _channel.invokeMethod('startPreview', {
      'canvas': canvas?.toMap() ?? {},
    });
  }

  Future<void> startPublishingStream(String url) async {
    return await _channel.invokeMethod('startPublishingStream', {
      'url': url,
    });
  }

  Future<void> stopPublishingStream() async {
    return await _channel.invokeMethod('stopPublishingStream');
  }

  Future<void> stopPreview() async {
    return await _channel.invokeMethod('stopPreview', {'channel': 0});
  }

  Future<void> startPlayingStream(String url, {NDCanvas canvas}) async {
    return await _channel.invokeMethod('startPlayingStream', {
      'url': url,
      'canvas': canvas?.toMap() ?? {},
    });
  }

  Future<void> stopPlayingStream() async {
    return await _channel.invokeMethod('stopPlayingStream');
  }

  /* EventHandler */

  static void _registerEventHandler() async {
    _streamSubscription = _event.receiveBroadcastStream().listen(_eventListener);
  }

  static void _unregisterEventHandler() async {
    await _streamSubscription?.cancel();
    _streamSubscription = null;
  }

  static void _eventListener(dynamic data) {
    final Map<dynamic, dynamic> map = data;
    switch(map['method']) {
      case 'onEventCallback':
        if (NodeMediaClientEngine.onEventCallback == null) return;
        NodeMediaClientEngine.onEventCallback(
            map['event'],
            map['msg']
        );
        break;
    }
  }
}
